<?php
/**
 * Copyright (C) 2025  Jaap Jansma (jaap.jansma@civicoop.org)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

namespace Civi\Api4\Service\Links;

use Civi\API\Event\RespondEvent;
use Civi\Core\Event\GenericHookEvent;
use CRM_Documents_ExtensionUtil as E;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class DocumentLinks implements EventSubscriberInterface {
  use LinksProviderTrait;


  /**
   * Returns an array of event names this subscriber wants to listen to.
   *
   * The array keys are event names and the value can be:
   *
   *  * The method name to call (priority defaults to 0)
   *  * An array composed of the method name to call and the priority
   *  * An array of arrays composed of the method names to call and respective
   *    priorities, or 0 if unset
   *
   * For instance:
   *
   *  * ['eventName' => 'methodName']
   *  * ['eventName' => ['methodName', $priority]]
   *  * ['eventName' => [['methodName1', $priority], ['methodName2']]]
   *
   * The code must not depend on runtime state as it will only be called at compile time.
   * All logic depending on runtime state must be put into the individual methods handling the events.
   *
   * @return array<string, string|array{0: string, 1: int}|list<array{0: string, 1?: int}>>
   */
  public static function getSubscribedEvents()
  {
    return [
      'civi.api4.getLinks' => 'alterDocumentLinks',
      'civi.api.respond' => 'alterDocumentLinksResult',
    ];
  }

  public static function alterDocumentLinks(GenericHookEvent $e): void {
    if ($e->entity == 'Document') {
      $actions = [
        'download' => [
          'path' => '[download_url]',
          'text' => E::ts('Download'),
          'icon' => 'fa-download',
        ],
        'new_version' => [
          'path' => 'civicrm/documents/newversion?reset=1&action=add&id=[id]',
          'text' => E::ts('Upload new version'),
          'icon' => 'fa-upload',
          'target' => 'crm-popup'
        ],
        'edit' => [
          'path' => 'civicrm/documents/document?reset=1&action=add&&id=[id]&entity=&entity_id=',
          'text' => E::ts('Edit Document'),
          'icon' => 'fa-pencil',
          'target' => 'crm-popup'
        ],
        'view_versions' => [
          'path' => 'civicrm/documents/versions?reset=1&&id=[id]',
          'text' => E::ts('View versions'),
          'icon' => 'fa-list-ol',
          'target' => 'crm-popup'
        ],
        'delete' => [
          'path' => 'civicrm/documents/document?reset=1&action=delete&&id=[id]&entity=&entity_id=',
          'text' => E::ts('Delete Document'),
          'icon' => 'fa-trash',
          'target' => 'crm-popup'
        ],
      ];
      $addTemplate = [
        'api_action' => 'get',
        'ui_action' => '',
        'entity' => 'Document',
      ];
      self::addLinks($e->links, $actions, $addTemplate);
    }
  }

  public static function alterDocumentLinksResult(RespondEvent $e): void {
    $request = $e->getApiRequest();
    if ($request['version'] == 4 && $request->getEntityName() === 'Document' && is_a($request, '\Civi\Api4\Action\GetLinks')) {
      $links = (array) $e->getResponse();
      $documentId = $request->getValue('id');
      $downloadIndex = self::getActionIndex($links, 'download');

      //$e->getResponse()->exchangeArray(array_values($links));
    }
  }

  private static function addLinks(array &$newLinks, array $actions, array $addTemplate) {
    foreach ($actions as $action => $values) {
      $addTemplate['ui_action'] = $action;
      $addTemplate['path'] = $values['path'];
      $addTemplate['text'] = $values['text'];
      $addTemplate['icon'] = $values['icon'];
      $addTemplate['target'] = $values['target'] ?? '';
      $newLinks[] = $addTemplate;
    }
  }


}