<?php
/**
 * Copyright (C) 2025  Jaap Jansma (jaap.jansma@civicoop.org)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

namespace Civi\Api4\Service\Spec\Provider;

use Civi\Api4\Service\Spec\FieldSpec;
use Civi\Api4\Service\Spec\Provider\Generic\SpecProviderInterface;
use Civi\Api4\Service\Spec\RequestSpec;
use Civi\Core\Service\AutoService;
use CRM_Documents_ExtensionUtil as E;

class DocumentGetSpecProvider extends AutoService implements SpecProviderInterface {
  /**
   * @param \Civi\Api4\Service\Spec\RequestSpec $spec
   *
   * @return void
   */
  public function modifySpec(RequestSpec $spec)
  {
    $field = new FieldSpec('fid', $spec->getEntity(), 'Integer');
    $field->setLabel(E::ts('File ID'))
      ->setTitle(E::ts('File ID'))
      ->setColumnName('id')
      ->setInputType('Number')
      ->setDescription(E::ts('ID of the CiviCRM file entity'))
      ->setType('Extra')
      ->setReadonly(TRUE)
      ->addOutputFormatter([static::class, 'formatFileId']);
    $spec->addFieldSpec($field);
    $field = new FieldSpec('fcs', $spec->getEntity(), 'String');
    $field->setLabel(E::ts('File Checksum'))
      ->setTitle(E::ts('File Checksum'))
      ->setColumnName('id')
      ->setInputType('Text')
      ->setDescription(E::ts('Checksum of the CiviCRM file entity'))
      ->setType('Extra')
      ->setReadonly(TRUE)
      ->addOutputFormatter([static::class, 'formatFileChecksum']);
    $spec->addFieldSpec($field);
    $field = new FieldSpec('clean_name', $spec->getEntity(), 'String');
    $field->setLabel(E::ts('File Clean Name'))
      ->setTitle(E::ts('File Clean Name'))
      ->setColumnName('id')
      ->setInputType('Text')
      ->setDescription(E::ts('Clean file name of the CiviCRM file entity'))
      ->setType('Extra')
      ->setReadonly(TRUE)
      ->addOutputFormatter([static::class, 'formatFileCleanName']);
    $spec->addFieldSpec($field);
    $field = new FieldSpec('download_url', $spec->getEntity(), 'String');
    $field->setLabel(E::ts('File Download URL'))
      ->setTitle(E::ts('File Download URL'))
      ->setColumnName('id')
      ->setInputType('Text')
      ->setDescription(E::ts('Download URL of the CiviCRM file entity'))
      ->setType('Extra')
      ->setReadonly(TRUE)
      ->addOutputFormatter([static::class, 'formatFileUrl']);
    $spec->addFieldSpec($field);
  }

  /**
   * @param string $entity
   * @param string $action
   *
   * @return bool
   */
  public function applies(string $entity, string $action)
  {
    return $entity == 'Document' && $action === 'get';
  }

  public static function formatFileId(&$value, $row, $fieldDefn) {
    $docId = (int) $value;
    $value = '';
    if ($docId) {
      $versionId = static::getLatestVersionId($docId);
      $attachment = static::getAttachment($versionId);
      if (!empty($attachment)) {
        $value = $attachment['fileID'];
      }
    }
  }

  public static function formatFileChecksum(&$value, $row, $fieldDefn) {
    $docId = (int) $value;
    $value = '';
    if ($docId) {
      $versionId = static::getLatestVersionId($docId);
      $attachment = static::getAttachment($versionId);
      if (!empty($attachment)) {
        $value = \CRM_Core_BAO_File::generateFileHash($versionId, $attachment['fileID']);
      }
    }
  }

  public static function formatFileCleanName(&$value, $row, $fieldDefn) {
    $docId = (int) $value;
    $value = '';
    if ($docId) {
      $versionId = static::getLatestVersionId($docId);
      $attachment = static::getAttachment($versionId);
      if (!empty($attachment)) {
        $value = $attachment['cleanName'];
      }
    }
  }

  public static function formatFileUrl(&$value, $row, $fieldDefn) {
    $docId = (int) $value;
    $value = '';
    if ($docId) {
      $versionId = static::getLatestVersionId($docId);
      $attachment = static::getAttachment($versionId);
      if (!empty($attachment)) {
        $value = $attachment['url'];
        $value = (string) \Civi::url('frontend:/' . $value, 'a');
      }
    }
  }

  private static function getLatestVersionId(int $documentId): int {
    static $versions = [];
    if (!isset($versions[$documentId])) {
      $sql = "SELECT `id` FROM `civicrm_document_version` WHERE `document_id` = %1 ORDER BY `version` DESC LIMIT 1";
      $versions[$documentId] = \CRM_Core_DAO::singleValueQuery($sql, ['1' => array($documentId, 'Integer')]);
    }
    return $versions[$documentId];
  }

  private static function getAttachment(int $versionId): array {
    static $attachments = [];
    if (!isset($attachments[$versionId])) {
      $attachments[$versionId] = [];
      $_attachments = \CRM_Core_BAO_File::getEntityFile('civicrm_document_version', $versionId);
      if (!empty($_attachments)) {
        $attachments[$versionId] = reset($_attachments);
      }
    }
    return $attachments[$versionId];
  }


}
