<?php

namespace Civi\Documents\ActionProvider\Action;

use \Civi\ActionProvider\Action\AbstractAction;
use Civi\ActionProvider\Parameter\FileSpecification;
use Civi\ActionProvider\Parameter\OptionGroupSpecification;
use \Civi\ActionProvider\Parameter\ParameterBagInterface;
use \Civi\ActionProvider\Parameter\SpecificationBag;
use \Civi\ActionProvider\Parameter\Specification;
use \Civi\ActionProvider\Utils\CustomField;

use Civi\CaseArchive\Documents\Entity\Document;
use CRM_Documents_ExtensionUtil as E;

class UploadNewDocument extends AbstractAction {

  /**
   * Run the action
   *
   * @param ParameterBagInterface $parameters
   *   The parameters to this action.
   * @param ParameterBagInterface $output
   *   The parameters this action can send back
   * @return void
   */
  protected function doAction(ParameterBagInterface $parameters, ParameterBagInterface $output) {
    $contactIds = [];
    if (is_array($parameters->getParameter('contact_ids')) && count($parameters->getParameter('contact_ids'))) {
      $contactIds = $parameters->getParameter('contact_ids');
    }

    $document = $this->createDocument($parameters, $contactIds);
    $version = $document->getCurrentVersion();

    $file = $parameters->getParameter('file');
    $uploadNewOne = true;
    if (empty($file)) {
      $uploadNewOne = false;
    } elseif (isset($file['id'])) {
      $uploadNewOne = false;
    }
    try {
      if (isset($file['id'])) {
        civicrm_api3('Attachment', 'delete', array('id' => $file['id']));
      }
    } catch (\Exception $e) {
      // Do nothing
    }

    $content = '';
    if (isset($file['content'])) {
      $content = base64_decode($file['content']);
    } elseif (isset($file['url'])) {
      $content = file_get_contents($file['url']);
    }
    if (empty($content)) {
      return;
    }

    if ($uploadNewOne) {
      $this->uploadFileContent($parameters, $version, $content, $file['name'], $file['mime_type']);
    }

    $output->setParameter('document_id', $document->getId());
  }

  protected function createDocument(ParameterBagInterface $parameters, array $contactIds): \CRM_Documents_Entity_Document {
    $documentsRepo = \CRM_Documents_Entity_DocumentRepository::singleton();
    $document = new \CRM_Documents_Entity_Document();
    if ($parameters->getParameter('subject')) {
      $document->setSubject($parameters->getParameter('subject'));
    } elseif ($this->configuration->getParameter('subject')) {
      $document->setSubject($this->configuration->getParameter('subject'));
    }
    if ($this->configuration->getParameter('type')) {
      $document->setTypeId($this->configuration->getParameter('type'));
    }
    if ($this->configuration->getParameter('status')) {
      $document->setStatusId($this->configuration->getParameter('status'));
    }
    if (count($contactIds)) {
      $document->setContactIds($contactIds);
    }
    if ($parameters->getParameter('case_id')) {
      $case = civicrm_api3('Case', 'getsingle', array("case_id"=>$parameters->getParameter('case_id')));
      $document->addCaseid($parameters->getParameter('case_id'));
      if (is_array($case['client_id'])) {
        foreach($case['client_id'] as $client_id) {
          $document->addContactId($client_id);
        }
      } else {
        $document->addContactId($case['client_id']);
      }

    }
    $version = $document->addNewVersion();
    $version->setDescription($parameters->getParameter('description'));
    $documentsRepo->persist($document);
    return $document;
  }

  protected function uploadFileContent(ParameterBagInterface $parameters, \CRM_Documents_Entity_DocumentVersion $version, string $content, string $cleanFileName, string $mimeType) {
    $config = \CRM_Core_Config::singleton();
    $directoryName = $config->customFileUploadDir;
    $fileName = \CRM_Utils_File::makeFileName($cleanFileName);
    $uri = $fileName;
    \CRM_Utils_File::createDir($directoryName);
    $fileParams = [
      'name' => $cleanFileName,
      'mime_type' => $mimeType,
      'uri' => $uri,
    ];
    if ($parameters->doesParameterExists('description')) {
      $fileParams['description'] = $parameters->getParameter('description');
    }
    $fileDao = \CRM_Core_BAO_File::create($fileParams);
    $fileDao->find(TRUE);

    $entityFileDao = new \CRM_Core_DAO_EntityFile();
    $entityFile['entity_table'] = 'civicrm_document_version';
    $entityFile['entity_id'] = $version->getId();
    $entityFileDao->copyValues($entityFile);
    $entityFileDao->file_id = $fileDao->id;
    $entityFileDao->save();

    $path = $config->customFileUploadDir . $uri;
    if (is_string($content)) {
      file_put_contents($path, $content);
    }
  }

  /**
   * Returns the specification of the configuration options for the actual action.
   *
   * @return SpecificationBag
   */
  public function getConfigurationSpecification() {
    return new SpecificationBag([
      new Specification('subject', 'String', E::ts('Subject'), false),
      new OptionGroupSpecification('type', 'document_type', E::ts('Document Type'), false),
      new OptionGroupSpecification('status', 'document_status', E::ts('Document Status'), false),
    ]);
  }

  /**
   * Returns the specification of the parameters of the actual action.
   *
   * @return SpecificationBag
   */
  public function getParameterSpecification() {
    $specs = new SpecificationBag();
    $specs->addSpecification(new Specification('subject', 'String', E::ts('Subject'), false));
    $specs->addSpecification(new Specification('contact_ids', 'Integer', E::ts('Contact IDs'), false, null, null, null, true));
    $specs->addSpecification(new Specification('case_id', 'Integer', E::ts('Case ID'), false, null, null, null, false));
    $specs->addSpecification(new FileSpecification('file', E::ts('File'), false));
    $specs->addSpecification(new Specification('description', 'String', E::ts('Description'), false));
    return $specs;
  }

  /**
   * Returns the specification of the output parameters of this action.
   *
   * This function could be overriden by child classes.
   *
   * @return SpecificationBag
   */
  public function getOutputSpecification() {
    return new SpecificationBag([
      new Specification('document_id', 'Integer', E::ts('Document ID')),
    ]);
  }


}
