<?php

require_once 'documents.civix.php';

use Civi\Api4\Service\Links\DocumentLinks;
use Civi\Api4\Service\Spec\Provider\DocumentCreateSpecProvider;
use Civi\Api4\Service\Spec\Provider\DocumentVersionCreateSpecProvider;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Civi\Api4\Service\Spec\Provider\DocumentGetSpecProvider;
use CRM_Documents_ExtensionUtil as E;

/**
 * Implements hook_civicrm_container()
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_container/
 */
function documents_civicrm_container(ContainerBuilder $container) {
  $container->addCompilerPass(new \Civi\Documents\CompilerPass());
  $container->register(DocumentGetSpecProvider::class, DocumentGetSpecProvider::class)
    ->addTag('spec_provider')
  ;
  $container->register(DocumentCreateSpecProvider::class, DocumentCreateSpecProvider::class)
    ->addTag('spec_provider')
  ;
  $container->register(DocumentVersionCreateSpecProvider::class, DocumentVersionCreateSpecProvider::class)
    ->addTag('spec_provider')
  ;
  $container->register(DocumentLinks::class, DocumentLinks::class)
    ->setPublic(true)
    ->addTag('event_subscriber')
  ;
}

/**
 * Implementation of hook_civicrm_config
 *
 * @link http://wiki.civicrm.org/confluence/display/CRMDOC/hook_civicrm_config
 */
function documents_civicrm_config(&$config) {
  static $_alreadyRun = FALSE;
  _documents_civix_civicrm_config($config);
  if (!$_alreadyRun) {
    \Civi::dispatcher()->addListener('civi.dao.preDelete', 'documents_civicrm_preDelete');
  }
  $_alreadyRun = TRUE;
}

/**
 * Implementation of hook_civicrm_install
 *
 * @link http://wiki.civicrm.org/confluence/display/CRMDOC/hook_civicrm_install
 */
function documents_civicrm_install() {
  return _documents_civix_civicrm_install();
}

/**
 * Implementation of hook_civicrm_enable
 *
 * @link http://wiki.civicrm.org/confluence/display/CRMDOC/hook_civicrm_enable
 */
function documents_civicrm_enable() {
  return _documents_civix_civicrm_enable();
}

/**
 * Implements hook_civicrm_entityTypes().
 *
 * Declare entity types provided by this module.
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_entityTypes
 */
function documents_civicrm_entityTypes(&$entityTypes) {
  $entityTypes = array_merge($entityTypes, [
    [
      'name' => 'Document',
      'class' => 'CRM_Documents_DAO_Document',
      'table' => 'civicrm_document',
    ],
    [
      'name' => 'DocumentVersion',
      'class' => 'CRM_Documents_DAO_DocumentVersion',
      'table' => 'civicrm_document_version',
    ],
    [
      'name' => 'DocumentEntity',
      'class' => 'CRM_Documents_DAO_DocumentEntity',
      'table' => 'civicrm_document_entity',
    ],
    [
      'name' => 'DocumentContact',
      'class' => 'CRM_Documents_DAO_DocumentContact',
      'table' => 'civicrm_document_contact',
    ],
    [
      'name' => 'DocumentCase',
      'class' => 'CRM_Documents_DAO_DocumentCase',
      'table' => 'civicrm_document_case',
    ],
  ]);
}

/**
 * Implementation of hook_civicrm_navigationMenu
 *
 * @link http://wiki.civicrm.org/confluence/display/CRMDOC/hook_civicrm_navigationMenu
 */
function documents_civicrm_navigationMenu( &$params ) {
  $item = array (
    "name"=> E::ts('Find documents'),
    "url"=> "civicrm/documents/search",
    "permission" => "administer CiviCRM",
    "weight" => 5,
  );
  _documents_civix_insert_navigation_menu($params, "Search", $item);
}

/**
 * Implementation of hook_civicrm_tabset
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_tabset
 */
function documents_civicrm_tabset($path, &$tabs, $context) {
  if ($path === 'civicrm/contact/view') {
    // add a tab to the contact summary screen
    $url = CRM_Utils_System::url('civicrm/contact/view/documents', ['cid' => $context['contact_id']]);

    //Count number of documents
    $documentRepo = CRM_Documents_Entity_DocumentRepository::singleton();
    $DocumentCount = count($documentRepo->getDocumentsByContactId($context['contact_id'], FALSE));

    $tabs[] = array(
      'id' => 'contact_documents',
      'url' => $url,
      'count' => $DocumentCount,
      'title' => E::ts('Documents'),
      'weight' => 1,
      'icon' => 'crm-i fa-files-o',
    );
  }
}

/**
 * Display the documents linked to a case
 *
 * Implementatio of hook_civicrm_caseSummary
 * @link http://wiki.civicrm.org/confluence/display/CRMDOC/hook_civicrm_caseSummary
 */
//function documents_civicrm_caseSummary($caseId) {
//  return array('documents_summary_section' => array('value' => $content, 'label' => E::ts('Documents')));
//}

/**
 * Add the searckit search to the case summary
 *
 * @param $page
 *
 * Implementation of hook_civicrm_pageRun
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_pageRun/
 */
function documents_civicrm_pageRun($page) {
  if ($page instanceof CRM_Case_Page_Tab
    && ($page->getVar('_action') & CRM_Core_Action::VIEW)
   ) {
    $caseId = $page->getVar('_id');
    $page = new CRM_Documents_Page_CaseDocuments($caseId);
    $content = $page->run();
    CRM_Core_Region::instance('page-body')->add([
      'markup' => $content,
    ]);
  }
}

/**
 * Removes the contact from a document as soon as a contact is deleted permanently.
 *
 * @param type $op
 * @param type $objectName
 * @param type $id
 * @param type $params
 */
function documents_civicrm_pre( $op, $objectName, $id, &$params ) {
  $repo = CRM_Documents_Entity_DocumentRepository::singleton();
  if ($objectName == 'Individual' || $objectName == 'Household' || $objectName == 'Organization') {
    if ($op == 'delete') {
      try {
        $contact = civicrm_api3('Contact', 'getsingle', array('contact_id' => $id, 'is_deleted' => '1'));
        //contact is in trash so this deletes the contact permanenty
        /** @var \CRM_Documents_Entity_Document[] $docs */
        $docs = $repo->getDocumentsByContactId($id);
        foreach($docs as $doc) {
          $doc->removeContactId($id);
          if ($doc->isAttached()) {
            $repo->persist($doc);
          } else {
            $repo->remove($doc);
          }
        }
      } catch (Exception $e) {
        //contact not found, or contact is in transh
      }
    }
  }

  if ($op == 'delete') {
    $refspec = CRM_Documents_Utils_EntityRef::singleton();
    $ref = $refspec->getRefByObjectName($objectName);
    if ($ref) {
      $documents = $repo->getDocumentsByEntityId($ref->getEntityTableName(), $id);
      foreach($documents as $doc) {
        $entity = new CRM_Documents_Entity_DocumentEntity($doc);
        $entity->setEntityId($id);
        $entity->setEntityTable($ref->getEntityTableName());
        $doc->removeEntity($entity);

        $repo->persist($doc);
      }
    }
  }
}

function documents_civicrm_preDelete(\Civi\Core\DAO\Event\PreDelete $event) {
  if ($event->object instanceof CRM_Case_DAO_Case) {
    $repo = CRM_Documents_Entity_DocumentRepository::singleton();
    /** @var \CRM_Documents_Entity_Document[] $docs */
    $docs = $repo->getDocumentsByCaseId($event->object->id);
    $repo->removeMultipleDocuments($docs);
  }
}

function documents_civicrm_postSave_civicrm_case($dao) {
  $repo = CRM_Documents_Entity_DocumentRepository::singleton();
  if (!$dao->id) {
    return;
  }
  $case = civicrm_api('Case', 'getsingle', array('id' => $dao->id, 'version' => 3));

  $docs = $repo->getDocumentsByCaseId($dao->id);
  foreach($docs as $doc) {
    //only remove the contacts from the document because the case is never completly deleted
    //so that we can set the contacts on a restore of a case
    foreach($case['client_id'] as $cid) {
      if ($case['is_deleted']) {
        $doc->removeContactId($cid);
      } else {
        $doc->addContactId($cid);
      }
    }
    $repo->persist($doc);
  }
}

/**
 * This hook is available through a patch from issue #CRM-14409
 *
 * @link https://issues.civicrm.org/jira/browse/CRM-14409
 *
 * @param type $mainContactId
 * @param type $mainCaseId
 * @param type $otherContactId
 * @param type $otherCaseId
 * @param type $changeClient
 */
function documents_civicrm_post_case_merge($mainContactId, $mainCaseId = NULL, $otherContactId = NULL, $otherCaseId = NULL, $changeClient = FALSE) {
  $repo = CRM_Documents_Entity_DocumentRepository::singleton();
  if (!empty($mainCaseId) && !empty($otherCaseId)) {
    $docs = $repo->getDocumentsByCaseId($otherCaseId);
    $case = civicrm_api('Case', 'getsingle', array('id' => $otherCaseId, 'version' => 3));
    foreach($docs as $doc) {
      $doc->addCaseId($mainCaseId);
      if ($changeClient) {
        $doc->removeCaseId($otherCaseId); //remove the old case
      }
      foreach($case['client_id'] as $cid) {
        $doc->addContactId($cid);
      }
      $repo->persist($doc);
    }
  }
}
